

#-*- coding:UTF-8 -*-
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#


"""Fake SMTP protocol


"""



# System imports
import time
import sys

# HoneyPython imports
from Honey.Protocol import LineProtocol




class FakeSMTP(LineProtocol):
    """Define the SMTP protocol

    """
    def sendlines(self, lines):
        """Helper method

        if lines is a string, send it
        if lines is a tuple or list, send each string

        """
        
        if isinstance(lines, (tuple, list)):
            for line in lines:
                self.sendline(line % self.attrs)
        else:
            self.sendline(lines % self.attrs)
     
    def connectionEstablished(self):
        """Callback

        called when client connects

        """
        
        self.sendlines(strings.CONN_ESTABLISHED)
        self.helo = False
        self.sender = None
        self.users = strings.USERS
        self.recipients = []
        self.data = False
        self.buffer = ''

    def connectionLost(self):
        """Callback

        called upon disconnection

        """
        
        self.sendlines(strings.CONN_LOST)

    def lineReceived(self, line):
        """Callback

        called when client sends data

        """

        # writing message ?
        if self.data:
            # if it's the end of message...
            if line.strip() == '.':
                self.sendlines(strings.MSG_ACCEPTED)
                self.reset()
                return
            # else, append the line to the buffer
            self.buffer += line
            return
            
        tokens = line.split()
        if len(tokens) == 0:
            self.do_ERROR('', [])
            return
        if len(tokens) == 1:
            action = tokens[0]
            args = []
        elif len(tokens) > 1:
            action = tokens[0]
            args = tokens[1:]

        self.attrs['action'] = action.lower()

        try:
            m = getattr(self, "do_" + action.upper())
        except AttributeError:
            self.do_ERROR(action, args)
            return

        m(args)

    def reset(self):
        """Called to reset session state, after sending message

        """
        
        self.data = False
        self.sender = None
        self.recipients = []
        self.buffer = ''

    def do_HELO(self, args):
        """Callback

        called upon HELO command

        """
        
        if not args:
            self.sendlines(strings.HELO_FAIL)
            return
        
        self.sendlines(strings.HELO_OK)
        self.helo = True


    def do_EHLO(self, args):
        """Callback

        called upon EHLO command

        """

        if not args:
            self.sendlines(strings.EHLO_FAIL)
            return
        
        self.sendlines(strings.EHLO_OK)
        self.helo = True

    def do_MAIL(self, args):
        """Callback

        called upon MAIL command

        """

        if not self.helo:
            self.sendlines(strings.MAIL_NOT_HELO)
            return
        
        if self.sender:
            self.sendlines(strings.MAIL_SENDER_SPECIFIED)
            return

        if not args:
            self.attrs['arg'] = '""'
            self.sendlines(strings.MAIL_INVALID_ARGS)
            return

        if not len(args) > 1 or not args[0] == 'from:':
            self.attrs['arg'] = args[0].strip(":")
            self.sendlines(strings.MAIL_INVALID_ARGS)
            return
        
        if '@' not in args[1]:
            self.attrs['arg'] = args[1]
            self.sendlines(strings.MAIL_INVALID_DOMAIN)
            return

        self.attrs['arg'] = args[1]
        self.sendlines(strings.MAIL_SENDER_OK)

        self.sender = args[1]

    def do_RCPT(self, args):
        """Callback

        called upon RCPT command

        """

        if not self.sender:
            self.sendlines(strings.RCPT_NEED_MAIL)
            return

        if not args:
            self.attrs['arg'] = '""'
            self.sendlines(strings.RCPT_INVALID_ARGS)
            return

        if not len(args) > 1 or not args[0] == 'to:':
            self.attrs['arg'] = args[0].strip(':')            
            self.sendline(strings.RCPT_INVALID_ARGS)
            return

        if args[1] not in self.users:
            self.attrs['arg'] = args[1]
            self.sendlines(strings.RCPT_USER_UNKNOWN)
            return

        self.attrs['arg'] = args[1]
        self.sendlines(strings.RCPT_RECIPIENT_OK)

        if not args[1] in self.recipients:
            self.recipients.append(args[1])

    def do_DATA(self, args):
        """Callback

        called upon DATA command

        """

        if not self.sender:
            self.sendlines(strings.DATA_NEED_MAIL)
            return
        
        if not self.recipients:
            self.sendlines(strings.DATA_NEED_RECIPIENT)
            return
        
        self.sendlines(strings.DATA_OK)
        self.data = True

    def do_RSET(self, args):
        """Callback

        called upon RSET command

        """

        self.sendlines(strings.RESET)
        self.reset()

    def do_NOOP(self, args):
        """Callback

        called upon NOOP command

        """

        self.sendlines(strings.NOOP)

    def do_QUIT(self, args):
        """Callback

        called upon QUIT command

        """

        self.sendlines(strings.QUIT)
        self.close()

    def do_HELP(self, args):
        """Callback

        called upon HELP command

        """

        if not args:
            self.sendlines(strings.HELP_NO_ARGS)
            return

        arg = ' '.join(args)

        try:
            s = getattr(strings, "HELP_%s" % arg.upper())
            self.sendlines(s)
        except AttributeError:
            self.attrs['arg'] = arg
            self.sendlines(strings.HELP_INVALID_TOPIC)


    def do_VRFY(self, args):
        """Callback

        called upon VRFY command

        """

        self.sendlines(strings.VRFY)

    def do_EXPN(self, args):
        """Callback

        called upon EXPN command

        """

        self.sendlines(strings.EXPN)

    def do_VERB(self, args):
        """Callback

        called upon VERB command

        """

        self.sendlines(strings.VERB)

    def do_ETRN(self, args):
        """Callback

        called upon ETRN command

        """

        if not args:
            self.sendlines(strings.ETRN_NO_ARGS)
            return

    def do_DSN(self, args):
        """Callback

        called upon DSN command

        """

        self.do_ERROR("dsn", args)

    def do_AUTH(self, args):
        """Callback

        called upon AUTH command

        """

        self.do_ERROR("auth", args)

    def do_STARTTLS(self, args):
        """Callback

        called upon STARTTLS command

        """

        self.do_ERROR("starttls", args)

    def do_ERROR(self, cmd, args):
        """Callback

        called on any invalid command error

        """

        self.attrs['arg'] = '"%s"'%cmd
        self.sendlines(strings.ERROR)


