
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""
# System imports
import sys
import time


class TaskQueue(object):
    def __init__(self):
        self.tasks = {}

    def __contains__(self, key):
        return key in self.tasks

    def put(self, delay, call, *args, **kwds):
        if not callable(call):
            raise TypeError("call 'must' be a callable object")

        if delay < 0:
            raise ValueError("delay can't be negative")
        
        deadline = time.time() + delay
        self.tasks[deadline] = (call, args, kwds)
        return deadline

    def remove(self, id):
        del self.tasks[id]

    def get(self, id):
        return self.tasks.pop(id)

    def expired(self):
        now = time.time()
        deadlines = self.tasks.keys(); deadlines.sort()
        assert type(deadlines) is list
        for deadline in deadlines:
            if deadline > now:
                raise StopIteration
            yield self.tasks.pop(deadline)

    def __iter__(self):
        return self.expired()
        

class Engine(object):

    timeout = 0.001

    def __init__(self):
        self.queue = TaskQueue()
        self.running = False

    def stop(self):
        self.running = False

    def run(self):
        self.running = True
        while self.running:
            self.update()

    def register(self, delay, call, *args, **kwds):
        return self.queue.put(delay, call, *args, **kwds)

    def unregister(self, id):
        self.queue.remove(id)

    def update(self):
        for call, args, kwds in self.queue:
            try:
                call(*args, **kwds)
            except:
                exc = sys.exc_info()
                self.handleException(*exc)

        time.sleep(self.timeout)

    def handleException(self, exc, value=None, tb=None):
        raise exc, value, tb

    

                
