
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""


class BaseProtocol(object):
    """Base class for protocols

    """
    def connectionEstablished(self):
        """Callback called when the connection is established

        """

        raise NotImplementedError

    def connectionLost(self):
        """Callback called when the connection is lost

        """

        raise NotImplementedError


class RawProtocol(BaseProtocol):
    """Receives and send raw data

    """

    def dataReceived(self, data):
        """Callback called to process received data

        """

        raise NotImplementedError


class LineProtocol(RawProtocol):
    """Line oriented protocol. Send and receive data line by line


    """
    
    lineMode = True
    _rbuffer = ''
    delimiter = '\r\n'
    MAX = 16384

    def dataReceived(self, data):
        """Process received data into lines

        """
        
        self._rbuffer = self._rbuffer + data
        while self.lineMode:
            try:
                line, self._rbuffer = self._rbuffer.split(self.delimiter, 1)
            except ValueError:
                if len(self._rbuffer) > self.MAX:
                    line, self._rbuffer = self._rbuffer, ''
                    self.lineLengthExceeded(line)
                    return
                break
            else:
                linelength = len(line)
                if linelength > self.MAX:
                    line, self._rbuffer = self._rbuffer, ''
                    self.lineLengthExceeded(line)
                    return
                self.lineReceived(line)

        else:
            data, self._rbuffer = self._rbuffer, ''
            if data:
                self.rawDataReceived(data)

    def setLineMode(self, extra=''):
        """Enter line mode

        """
        
        self.lineMode = True
        self.dataReceived(extra)

    def setRawMode(self):
        """Enter Raw mode

        """
        
        self.lineMode = False

    def rawDataReceived(self, data):
        """Callback

        Should be overriden
        
        """
        
        raise NotImplementedError

    def lineReceived(self, line):
        """Callback

        Should be overriden
        
        """
        
        raise NotImplementedError

    def sendline(self, line):
        """Send a line. Use this method instead of 'send'

        """

        self.send(line + self.delimiter)

    def lineLengthExceeded(self, line):
        """Callback called when a line exceeeds max length

        This default implementation simply close the socket.

        Override it if this is not the desired behavior

        """

        self.close()

