
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""

# System imports
import sys
sys.path.insert(0, "..")

import unittest

# HoneyPython imports

from Honey.Protocol import BaseProtocol, RawProtocol, LineProtocol


class MyBaseProtocol(BaseProtocol):
    def connectionEstablished(self):
        self.active = True

    def connectionLost(self):
        self.active = False


class TestBaseProtocol(unittest.TestCase):
    def setUp(self):
        self.protocol = MyBaseProtocol()
        self.protocol.connectionEstablished()

    def testActive(self):
        """Protocol instance is initialized"""
        self.assert_(self.protocol.active)

    def testInactive(self):
        """Protocol instance is shutdown"""
        self.protocol.connectionLost()
        self.assert_(not self.protocol.active)


class MyRawProtocol(RawProtocol):
    def connectionEstablished(self):
        self.active = True

    def connectionLost(self):
        self.active = False

    def dataReceived(self, data):
        self.data = data


class TestRawProtocol(TestBaseProtocol):
    def setUp(self):
        self.protocol = MyRawProtocol()
        self.protocol.connectionEstablished()

    def testDataReceived(self):
        """Protocol receives data"""
        self.protocol.dataReceived("I am a great one duh")
        self.assertEqual(self.protocol.data, "I am a great one duh")

class LineLengthExceeded(ValueError):
    pass

class MyLineProtocol(LineProtocol):
    line = ""
    def connectionEstablished(self):
        self.active = True

    def connectionLost(self):
        self.active = False
    
    def rawDataReceived(self, data):
        self.data = data

    def lineReceived(self, line):
        self.line = line

    def send(self, line):
        self.sent = line

    def lineLengthExceeded(self, line):
        raise LineLengthExceeded(line)
    

class TestLineProtocol(TestBaseProtocol):
    def setUp(self):
        self.protocol = MyLineProtocol()
        self.protocol.connectionEstablished()

    def testDataReceived(self):
        """Protocol receives data in line mode"""
        self.protocol.dataReceived("I am a great one duh")
        self.assertEqual(self.protocol.line, "")
        self.protocol.dataReceived("\r\n and this will not be processed !")
        self.assertEqual(self.protocol.line, "I am a great one duh")

    def testRawMode(self):
        """Protocol receives data in raw mode"""
        data = "Index\r\nLibrorum\r\nProibitorum\r\n"
        self.protocol.setRawMode()
        self.assert_(not self.protocol.lineMode)
        self.protocol.dataReceived(data)
        self.assertEqual(self.protocol.data, data)

    def testLineOverflow(self):
        """Crashes when line length overflows"""
        limit = self.protocol.MAX
        line = "and what" + ("?" * limit)
        self.assertRaises(LineLengthExceeded, self.protocol.dataReceived, line)
        
    

if __name__ == "__main__":
    unittest.main()
