
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""

# System imports
import sys
#if ".." not in sys.path:
#    sys.path.insert(0, "..")
import socket

import unittest

# HoneyPython imports

from Honey.Socket import ServerSocket, ClientSocket
from Honey.Selector import Select, _reads, _writes, READ, WRITE
from Honey.Protocol import RawProtocol

# Exceptions
from Honey.Error import SelectError

class FakeServer(object):
    host = socket.gethostname()
    def _close(self, client, reason=None):
        client._close(reason)

HOST = socket.gethostname()
PORT = 65004
SERVER = FakeServer()

class MyProtocol(RawProtocol):
    def connectionEstablished(self):
        pass

    def dataReceived(self, data):
        pass

    def connectionLost(self):
        pass


class FakeSocket(object):
    def __init__(self, fileno=None):
        self.fileno = fileno or 65536

    def recv(self, bufsize):
        return ""

    def send(self, data):
        return 0

    def close(self):
        pass


class ClientTester(object):
    def __init__(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.connect((HOST, PORT))
        self.close = self.socket.close

    def trySend(self):
        sent = self.socket.send("trying to send data")
        return sent

    def tryRecv(self):
        data = self.socket.recv(8192)
        return data


class TestSelect(unittest.TestCase):

    def setUp(self):
        self.select = Select()
        self.server = ServerSocket(SERVER, MyProtocol)
        self.server.bind(HOST, PORT)
        self.server.listen()

    def tearDown(self):
        self.server._close()
        del _reads[:]
        del _writes[:]

    def testRegisterSocket(self):
        self.select.register(self.server, READ)
        self.select.register(self.server, WRITE)
        self.assert_(self.server in _reads)
        self.assert_(self.server in _writes)

    def testRegisterInvalidSocket(self):
        socket = FakeSocket()
        self.assertRaises(SelectError, self.select.register, socket, READ)
        self.assertRaises(SelectError, self.select.register, socket, WRITE)

    def testRegisterSocketWithNoFileno(self):
        server = self.server
        del server.fileno
        self.assertRaises(SelectError, self.select.register, server, READ)
        self.assertRaises(SelectError, self.select.register, server, WRITE)

    def testRegisterClosedSocket(self):
        server = self.server
        server.closed = True
        self.assertRaises(SelectError, self.select.register, server, READ)
        self.assertRaises(SelectError, self.select.register, server, WRITE)

    def testReRegisterSocket(self):
        server = self.server
        self.select.register(server, 0)
        self.select.register(server, 1)

        self.assertRaises(SelectError, self.select.register, server, READ)
        self.assertRaises(SelectError, self.select.register, server, WRITE)

    def testUnregisterSocket(self):
        self.select.register(self.server, READ)
        self.select.register(self.server, WRITE)
        self.select.unregister(self.server, READ)
        self.select.unregister(self.server, WRITE)
        self.assert_(self.server not in _reads)
        self.assert_(self.server not in _writes)

    def testUnregisterNonExistantSocket(self):
        server = self.server
        self.assertRaises(SelectError, self.select.unregister, server, READ)
        self.assertRaises(SelectError, self.select.unregister, server, WRITE)

    def testCheckSocketsI(self):
        socket1 = ServerSocket(SERVER, MyProtocol)
        self.select.register(socket1, READ)
        self.select.register(socket1, WRITE)
        socket2 = ServerSocket(SERVER, MyProtocol)
        self.select.register(socket2, READ)
        self.select.register(socket2, WRITE)
        del socket1.fileno
        self.select.checkSockets()
        self.assert_(socket1 not in _reads)
        self.assert_(socket1 not in _writes)
        self.assert_(socket2 in _reads)
        self.assert_(socket2 in _writes)

    def testCheckSocketsII(self):
        socket1 = ServerSocket(SERVER, MyProtocol)
        self.select.register(socket1, READ)
        self.select.register(socket1, WRITE)
        socket2 = ServerSocket(SERVER, MyProtocol)
        self.select.register(socket2, READ)
        self.select.register(socket2, WRITE)
        socket1.close()
        self.select.checkSockets()
        self.assert_(socket1 not in _reads)
        self.assert_(socket1 not in _writes)
        self.assert_(socket2 in _reads)
        self.assert_(socket2 in _writes)

    def testClean(self):
        socket1 = ServerSocket(SERVER, MyProtocol)
        self.select.register(socket1, READ)
        self.select.register(socket1, WRITE)
        socket2 = ServerSocket(SERVER, MyProtocol)
        self.select.register(socket2, READ)
        self.select.register(socket2, WRITE)
        self.select.clean()
        self.assert_(not _reads)
        self.assert_(not _writes)

    def testDoSelect(self):
        server = self.server
        self.select.register(server, READ)
        remote = ClientTester()
        reads, writes = self.select.doSelect()
        self.assert_(server in reads and server not in writes)
        local = server.accept()

        self.select.register(local, READ)
        self.select.register(local, WRITE)

        reads, writes = self.select.doSelect()
        self.assert_(server not in reads and server not in writes)
        self.assert_(local not in reads and local in writes)

        remote.trySend()
        reads, writes = self.select.doSelect()
        self.assert_(server not in reads and server not in writes)
        self.assert_(local in reads and local in writes)
        data = local.recv()
        
        reads, writes = self.select.doSelect()
        self.assert_(server not in reads and server not in writes)
        self.assert_(local not in reads and local in writes)

        del local.fileno
        reads, writes = self.select.doSelect()
        self.assert_(local not in _reads)
        self.assert_(local not in _writes)
        self.assert_(not reads and not writes)

        
if __name__ == "__main__":
    unittest.main()

