
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""

# System imports
import sys
sys.path.insert(0, "..")

import socket
import unittest

# HoneyPython imports

from Honey.Server import TCPServer
from Honey.Protocol import RawProtocol
from Honey.Socket import BaseSocket
from Honey.Error import ServerError, ConnectionError
from Honey.Selector import Select, _reads, _writes
from Honey.Loop import Engine

HOST = socket.gethostname()
PORT = 65001


# Global used for client testing

clientData = {"sock":None, 'active':False, 'data':None}


class MyProtocol(RawProtocol):
    def connectionEstablished(self):
        clientData['sock'] = self
        clientData['active'] = True

    def connectionLost(self):
        clientData['active'] = False

    def dataReceived(self, data):
        clientData['data'] = data


class ClientTester(object):
    def __init__(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.connect((HOST, PORT))
        self.close = self.socket.close
        self.str = "Wake up... time to die..."
        
    def trySend(self):
        sent = self.socket.send(self.str)
        return sent

    def tryRecv(self):
        data = self.socket.recv(8192)
        return data


class TestBaseServer(unittest.TestCase):
    def setUp(self):
        self.server = TCPServer(HOST)

    def tearDown(self):
        clientData['active'] = False
        clientData['data'] = None
        clientData['sock'] = None
        self.server.clean()
        
    def testServerAttributes(self):
        self.assertEqual(self.server.host, HOST)
        self.assert_(issubclass(self.server.type, BaseSocket))
        self.assert_(isinstance(self.server.select, Select))
        self.assert_(isinstance(self.server.engine, Engine))


class TestServerBindListen(TestBaseServer):
    def testBindServerPort(self):
        self.server.bind(PORT, MyProtocol)
        self.assert_(MyProtocol in self.server.ports[PORT].protocol.__bases__)
        self.assert_(self.server.ports[PORT] in _reads)

    def testRebindServerPort(self):
        self.server.bind(PORT, MyProtocol)
        self.assertRaises(ServerError, self.server.bind, PORT, MyProtocol)

    def testBindInvalidProtocol(self):
        self.assertRaises(ServerError, self.server.bind, PORT, object)

    def testServerClean(self):
        self.server.bind(PORT, MyProtocol)
        self.server.clean()
        self.assert_(not self.server.ports)
        self.assert_(not _reads)
        self.assert_(not _writes)


class TestServerCallbacks(TestBaseServer):
    def setUp(self):
        super(TestServerCallbacks, self).setUp()
        self.server.bind(PORT, MyProtocol)
        self.client = ClientTester()

    def tearDown(self):
        super(TestServerCallbacks, self).tearDown()
        self.client.close()

class TestDirectCall(TestServerCallbacks):
    def setUp(self):
        super(TestDirectCall, self).setUp()
        self.server._accept(self.server.ports[PORT])
        self.assert_(clientData['active'])
        self.sock = clientData['sock']
        
    def testAcceptCallback(self):
        self.assert_(self.sock in _reads)
        self.assert_(self.sock in _writes)

    def testCloseClientCallback(self):
        self.server._close(self.sock)
        self.assert_(not self.sock)
        self.assert_(not clientData['active'])
        self.assert_(self.sock.closed)
        self.assert_(self.sock not in _reads)
        self.assert_(self.sock not in _writes)

    def testReadClientCallback(self):
        self.client.trySend()
        self.server._read(self.sock)
        self.assertEqual(clientData['data'], self.client.str) 

    def testWriteClientCallback(self):
        data = "...this piece of junk ?!?!"
        self.sock.send(data)
        self.server._write(self.sock)
        rdata = self.client.tryRecv()
        self.assertEqual(data, rdata)


class TestUpdateCall(TestServerCallbacks):
    def setUp(self):
        super(TestUpdateCall, self).setUp()
        self.server._update()
        self.sock = clientData['sock']
        
    def testAcceptOnUpdate(self):
        self.assert_(self.sock)
        self.assert_(self.sock in _reads)
        self.assert_(self.sock in _writes)
        self.assert_(clientData['active'])

    def testReadOnUpdate(self):
        self.client.trySend()
        self.server._update()
        self.assertEqual(clientData['data'], self.client.str)

    def testWriteOnUpdate(self):
        data = "...this piece of junk ?!?!"
        self.sock.send(data)
        self.server._update()
        rdata = self.client.tryRecv()
        self.assertEqual(data, rdata)

    def testRemoteClosed(self):
        self.client.close()
        self.server._update()
        self.assert_(not clientData['active'])

    def testDataFlushOnClose(self):
        data = "...this piece of junk ?!?!"
        self.sock.send(data)
        self.server._close(self.sock)
        self.server._update()
        rdata = self.client.tryRecv()
        self.assertEqual(data, rdata)
        

class TestServerRun(TestServerCallbacks):
    def setUp(self):
        super(TestServerRun, self).setUp()
        self.loop = self.server.engine.register

    def runAndStop(self):
        self.server.engine.register(1, self.server.stop)
        self.server.run()
        
    def testServerUpdating(self):
        self.runAndStop()
        self.assert_(clientData['active'])

    def _clientInitialized(self):
        self.assert_(clientData['active'])
        sock = clientData['sock']
        self.assert_(sock in _reads)
        self.assert_(sock in _writes)
    
    def testServerContinueUpdating(self):
        self.loop(0.1, self._clientInitialized)
        self.loop(0.2, self.client.close)
        self.runAndStop()
        self.assert_(not clientData['active'])
        sock = clientData['sock']
        self.assert_(sock not in _reads)
        self.assert_(sock not in _writes)
        
        
if __name__ == "__main__":
    unittest.main()
        
        
        
