
# Copyright (C) 2003  Pedro Werneck
#
# This library is free software; you can redistribute it and/or
# modify it under the terms of the GNU Lesser General Public
# License as published by the Free Software Foundation; either
# version 2.1 of the License, or (at your option) any later version.
#
# This library is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
# Lesser General Public License for more details.
#
# You should have received a copy of the GNU Lesser General Public
# License along with this library; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#

"""

"""

# System imports
import sys
sys.path.insert(0, "..")

import socket
import unittest

# HoneyPython imports

from Honey.Socket import BaseSocket, ServerSocket, ClientSocket
from Honey.Error import SocketError
from Honey.Protocol import RawProtocol


class FakeServer(object):
    host = ""
    def _close(self, client, reason=None):
        client._close(reason)


HOST = socket.gethostname()
PORT = 65004
SERVER = FakeServer()


class TestBaseSocket(unittest.TestCase):
    def setUp(self):
        self.socket = BaseSocket(SERVER)

    def tearDown(self):
        self.socket.close()

    def testAttributes(self):
        """Attributes are sane"""
        self.assert_(hasattr(self.socket, "fileno"))
        self.assert_(hasattr(self.socket, "socket"))
        self.assert_(hasattr(self.socket, "closed"))
        self.assert_(hasattr(self.socket, "family"))
        self.assert_(hasattr(self.socket, "type"))
        self.assert_(hasattr(self.socket, "attrs"))
        self.assertEqual(self.socket.fileno, self.socket.socket.fileno)

    def testCloseSocket(self):
        """Socket closes"""
        self.socket.close()
        self.assert_(self.socket.closed)

    def testBool(self):
        """Truth testing on socket state"""
        self.assert_(self.socket)
        self.socket.close()
        self.assert_(not self.socket)

    def testServerWeakRef(self):
        """Weak-reference to server managed by 'property'"""
        self.assert_(self.socket.server is SERVER)


class ClientTester(object):
    def __init__(self):
        self.socket = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        self.socket.connect((HOST, PORT))
        self.close = self.socket.close

    def trySend(self):
        sent = self.socket.send("trying to send data")
        return sent

    def tryRecv(self):
        data = self.socket.recv(8192)
        return data


class DataReceived(Exception):
    pass

class ConnectionLost(Exception):
    pass


class DumbProtocol(RawProtocol):
    def dataReceived(self, data):
        raise DataReceived(data)

    def connectionEstablished(self):
        self._established = True

    def connectionLost(self):
        raise ConnectionLost


class TestServerSocket(TestBaseSocket):#(unittest.TestCase):
    def setUp(self):
        self.socket = ServerSocket(SERVER, DumbProtocol)

    def tearDown(self):
        self.socket.close()

    def testAttributes(self):
        """ServerSocket attributes are sane"""
        self.assert_(hasattr(self.socket, "fileno"))
        self.assert_(hasattr(self.socket, "socket"))
        self.assert_(hasattr(self.socket, "closed"))
        self.assert_(hasattr(self.socket, "family"))
        self.assert_(hasattr(self.socket, "type"))
        self.assert_(hasattr(self.socket, "addr"))
        self.assert_(hasattr(self.socket, "attrs"))
        self.assert_(hasattr(self.socket, "name"))
        self.assert_(hasattr(self.socket, "protocol"))
        self.assertEqual(self.socket.fileno, self.socket.socket.fileno)

    def testBindSocket(self):
        """Socket binds to given host and port"""
        self.socket.bind(HOST, PORT)
        self.assertRaises(SocketError, self.socket.bind, HOST, PORT)
        
    def testListenSocket(self):
        """Socket starts listening"""
        self.socket.bind(HOST, PORT)
        self.socket.listen()
        client = ClientTester()
        client.close()

    def testAcceptSocket(self):
        """Client is accepted and wrapped"""
        self.socket.bind(HOST, PORT)
        self.socket.listen()
        client = ClientTester()
        server = self.socket.accept()
        
        self.assert_(isinstance(server, ClientSocket))
        client.close()
        self.assertRaises(ConnectionLost, server._close)



class TestClientSocket(unittest.TestCase):
    def setUp(self):
        self.server = ServerSocket(SERVER, DumbProtocol)
        self.server.bind(HOST, PORT)
        self.server.listen()
        self.remote = ClientTester()
        self.local = self.server.accept()
        self.socket = self.local

    def tearDown(self):
        self.remote.close()
        self.assertRaises(ConnectionLost, self.local.close)
        self.server.close()

    def testAttributes(self):
        """ClientSocket attributes"""
        self.assert_(hasattr(self.local, "fileno"))
        self.assert_(hasattr(self.local, "socket"))
        self.assert_(hasattr(self.local, "closed"))
        self.assert_(hasattr(self.local, "family"))
        self.assert_(hasattr(self.local, "type"))
        self.assert_(hasattr(self.local, "addr"))
        self.assert_(hasattr(self.local, "attrs"))
        self.assert_(hasattr(self.local, "_buffer"))
        self.assert_(self.local._established)
        self.assertEqual(self.local.attrs['host'], self.local.addr[0])
        self.assertEqual(self.local.attrs['port'], self.local.addr[1])

    def testRecvData(self):
        """Data is received"""
        self.remote.trySend()
        self.assertRaises(DataReceived, self.local.recv)

    def testRemoteClosed(self):
        """Local closes when remote drops"""
        self.remote.close()
        self.assertRaises(SocketError, self.local.recv)

    def testSend(self):
        """Data is sent"""
        data = "Once you start down the dark path..."
        rest = "...forever will it dominate your destiny"
        self.local.send(data)
        self.assertEqual(self.local._buffer, data)
        self.local.send(rest)
        self.assertEqual(self.local._buffer, data+rest)

    def testFlush(self):
        """Data is flushed"""
        data = "Once you start down the dark path, "\
               "forever will it dominate your destiny"
        self.local.send(data)
        self.assertEqual(self.local._buffer, data)
        sent = self.local.flush()
        self.assertEqual(self.local._buffer, data[len(sent):])
        recv = self.remote.tryRecv()
        self.assertEqual(recv, sent)




if __name__ == "__main__":
    unittest.main()
